#!/bin/bash
 
 
set -e
 
# Usage
function usage(){
 
if [ -z "$1" ] ; then
  echo "  "
  echo "Usage: "
  echo "   This is a tool for Monitor jvm information, only supports inceptor on TDC ."
  echo "   bash $0 inceptor1 jstack/jmap/jstat/jvm"
  echo "  "
  echo "Example:"
  echo "   If you want to Monitor jvm information of incepto-7jnkd in namespace ob2bzds, you need to use '7jnkd' 'ob2bzds' parameter. "
  echo "   bash $0 7jnkd ob2bzds"
  echo "   "
  exit 1
fi
}
 
ns=$2
 
# Package and email us
function package(){
if [ "`ls -A $_exp_dir`" = "" ]; then
  rm -rf $_exp_dir
  echo "No information was collected."
else
  tar -zcvf $_out_file $_exp_dir > /dev/null
  echo "`date '+%Y-%m-%d %H:%M:%S'` List all files in tar with tar -tvf $_info_dir/$_out_file."
  echo "`date '+%Y-%m-%d %H:%M:%S'` Email us at service@transwarp.io"
fi
}
 
# Validate aliasesi, Only in BASH
#source /etc/profile.d/kubernetes_master.sh >> /dev/null 2>&1
#shopt -s  expand_aliases
 
 
# Create WorkDir which be used to hold JVM file
_stat_date=`date '+%Y%m%d%H%M'`
_exp_dir=/tmp/jvm-$1-$_stat_date
_out_file=${_exp_dir}.tar.gz
 
# O Area usage
_o_ratio=80
echo "`date '+%Y-%m-%d %H:%M:%S'` The threshold of JVM Old's area is ${_o_ratio}"
 
 
if [ ! -d $_exp_dir ] ; then
  mkdir -m 755 -pv $_exp_dir
fi
 
 
user="hive"
echo "`date '+%Y-%m-%d %H:%M:%S'` $0 Monitoring memory usage of $1. "
 
 
function get_pid(){
# Get srv's pid
 
i=0
pod_arr=()
pods=`kubectl -n $ns get pod | grep -i $1 |grep -v hyperbase-thrift|  sort -r | cut -d ' ' -f 1 | tr '\n' ' '`
 
 
 for p_id in $pods;
     do
        pod_arr[$i]=$p_id,$(kubectl -n $ns exec -ti $p_id -- jps |grep -iE "InceptorServer2|RunJar|CoarseGrainedExecutorBackend" | awk '{printf "%s,%s",$1,$2}')
        let i+=1
     done
 
 
echo ${pod_arr[*]}
}
 
echo "`date '+%Y-%m-%d %H:%M:%S'` $0 Start Get srv's pid"
pod_arr=($(get_pid $1))
# pod_arr demo (hyperbase-regionserver-hyperbase1-66c4456dc-hlkk4,24,HRegionServer)
echo "`date '+%Y-%m-%d %H:%M:%S'` $0 Get srv's pid Finished"
 
 
 
# Get O Area usage
echo "`date '+%Y-%m-%d %H:%M:%S'` Get usage of JVM Old's area from Jstat"
j=0
jstat_arr=()
while [ $j -lt ${#pod_arr[*]} ]
do
    # echo ${pod_arr[$j]} | awk -F',' '{print $1,$2,$3}'
    tmp_arr=(`echo ${pod_arr[$j]} | awk -F',' '{print $1,$2}' `)
        # tmp_arr demo (hyperbase-regionserver-hyperbase1-66c4456dc-hlkk4,24)
    jstat_arr[$j]=${tmp_arr[0]},${tmp_arr[1]},$(kubectl -n $ns exec -ti ${tmp_arr[0]} -- jstat -gcutil ${tmp_arr[1]} 1000  5|tail -1 | awk '{printf ("%s",$4)}')
    # jstat_arr demo (hyperbase-regionserver-hyperbase1-66c4456dc-hlkk4,24,30.6)
        let j+=1
done
echo "`date '+%Y-%m-%d %H:%M:%S'` Get usage of JVM Old's area from Jstat Finished"
 
 
 
 
# Compase usage of JVM Old's area, If the usage is greater than _o_ratio, JVM information will collected
j=0
while  [[ $j -lt ${#jstat_arr[*]} ]]
do
    echo "`date '+%Y-%m-%d %H:%M:%S'` Start Compase usage of JVM Old's area"
    tmp_arr=(`echo ${jstat_arr[$j]} | awk -F',' '{print $1,$2,$3}' `)
        # tmp_arr demo (hyperbase-regionserver-hyperbase1-66c4456dc-hlkk4,24,30.6)
        usage_o=`echo ${tmp_arr[2]} | cut -d'.' -f 1`
        echo "`date '+%Y-%m-%d %H:%M:%S'` ${tmp_arr[0]}'s usage of JVM Old's area is ${tmp_arr[2]}"
    if [[ `echo ${tmp_arr[2]} | cut -d'.' -f 1` -gt ${_o_ratio:-80} ]]
    then
        i=1
        while [ $i -le 5 ]
        do
                    echo "`date '+%Y-%m-%d %H:%M:%S'` The usage of JVM Old's area is ${tmp_arr[3]} greater than ${_o_ratio}."
            echo "`date '+%Y-%m-%d %H:%M:%S'` Start get jstack of ${tmp_arr[0]}"
            kubectl -n $ns exec -ti ${tmp_arr[0]} -- sudo -u $user jstack ${tmp_arr[1]} >> $_exp_dir/${tmp_arr[0]}.jstack
            sleep 1s
            let i=i+1
                        echo "`date '+%Y-%m-%d %H:%M:%S'` Get jstack of ${tmp_arr[0]} Finished"
        done
        echo "`date '+%Y-%m-%d %H:%M:%S'` Start get jstat of ${tmp_arr[0]}"
        kubectl -n $ns exec -ti ${tmp_arr[0]} -- jstat -gcutil ${tmp_arr[1]} 1000 10 > $_exp_dir/${tmp_arr[0]}.jstat
                echo "`date '+%Y-%m-%d %H:%M:%S'` Get jstat of ${tmp_arr[0]} Finished"
                echo "`date '+%Y-%m-%d %H:%M:%S'` Get jmap of ${tmp_arr[0]} Finished"
        kubectl -n $ns exec -ti ${tmp_arr[0]} -- sudo -u $user jmap -histo:live ${tmp_arr[1]}  > $_exp_dir/${tmp_arr[0]}_`date +%Y%m%d%H%M`.jmap
                echo "`date '+%Y-%m-%d %H:%M:%S'` Get jmap of ${tmp_arr[0]} Finished"
    else
                echo "`date '+%Y-%m-%d %H:%M:%S'` The usage of JVM Old's area is ${tmp_arr[3]} not greater than ${_o_ratio}."
    fi
    let j+=1
done
 
 
if [ "`ls -A $_exp_dir`" = "" ]; then
  rm -rf $_exp_dir
  echo "`date '+%Y-%m-%d %H:%M:%S'` No information was collected."
else
  tar -zcvf $_out_file $_exp_dir > /dev/null
  echo "`date '+%Y-%m-%d %H:%M:%S'` List all files in tar with tar -tvf $_info_dir/$_out_file."
  echo "`date '+%Y-%m-%d %H:%M:%S'` Email us at service@transwarp.io"
fi